/* Copyright  2008 Vermilion Software ltd.  This software is the confidential and proprietary information of Vermilion Software ltd. (Confidential Information). You are not 
 * permitted to use, copy, disclose or broadcast such Confidential Information, save that Vermilion Software Ltd has permitted certain parties to use it on the terms of certain licence 
 * agreements.  If you are not party to such an agreement, you are requested to report the matter to Vermilion Software Ltd on +44 (0)845 241 9966.  Failure to do so may result in a criminal 
 * offence and legal liability to compensate Vermilion Software ltd.  Vermilion Software ltd will rigorously enforce all its rights in relation to the Confidential Information.
 */

/*
 * $Log: GeneralHelper.java,v $
 * Revision 1.10  2010/06/08 10:38:38  adam.cummings
 * Changes made to reflect the MT updates for 2.11 and more flexible security name processing
 *
 * Revision 1.9  2010/04/29 19:51:08  adam.cummings
 * Changed to accommodate manual DS for deletion
 *
 * Revision 1.8  2010/03/11 12:17:03  adam.cummings
 * Committed after update of Security Processor to accommodate DFA guidelines
 *
 * Revision 1.7  2010/02/22 12:37:53  adam.cummings
 * Solved 'S problem on security names
 *
 * Revision 1.6  2010/01/19 09:38:35  adam.cummings
 * Lots of security name updates done through the propercase method
 *
 * Revision 1.5  2010/01/14 16:36:56  adam.cummings
 * Committed after red warnings were removed elsewhere
 *
 * Revision 1.4  2010/01/07 17:01:49  adam.cummings
 * Checked in before testing the suppression of red marks
 *
 * Revision 1.3  2009/12/11 15:56:55  adam.cummings
 * Added a method and strings to take out of the security name
 *
 * Revision 1.2  2009/12/09 15:40:14  adam.cummings
 * Few changes
 *
 * Revision 1.1  2009/10/25 19:26:48  Alan.Beer
 * *** empty log message ***
 *
 * Revision 1.1  2009/09/24 21:52:01  Alan.Beer
 * *** empty log message ***
 *
 * Revision 1.1  2009/06/05 10:47:17  Alan.Beer
 * initial checkin
 *
 *
 */
package com.vermilion.vrs.common;

import java.math.BigDecimal;
import java.math.MathContext;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.vermilionsoftware.vrs.common.exception.VermilionNonFatalException;
import com.vermilionsoftware.vrs.dgs.api.ParameterRecord;

/**
 * Date Helper
 */

public class DateHelper {

    //  Date
    public static final String ENTITY_CD_DATE = "DATE";
    public static final String ENTITY_NM_DATE = "Date";
    public static final String ENTITYF_DATE_SYSDATE = "SysDate";
    
    //  Term
    public static final String ENTITY_CD_TERM = "TERM";
    public static final String ENTITY_NM_TERM = "Term";
    public static final String ENTITYF_TERM_LIST_NAME = "List Name";
    public static final String ENTITYF_TERM_CODE = "Code";
    public static final String ENTITYF_TERM_CODE_VARIABLE = "$_Term.Code$";
    
    //		 Terms.
    public static final String TERM_QTD = "QTD";
    public static final String TERM_MTD = "MTD";
    public static final String TERM_YTD = "YTD";
    public static final String TERM_FYTD = "FYTD";
    public static final String TERM_FMTD = "FMTD";
    public static final String TERM_FQTD = "FQTD";
    public static final String TERM_WTD = "WTD"; // Monday to Date
    public static final String TERM_6MTD = "6MTD"; // Half year to date (from Jan-1)
    public static final String TERM_SI = "SI";


    // --------------------------------------------------------------------------------------------------------------------------------------------------
    // General
    public static final double _ONE_THOUSAND = 1000;
    public static final double _ONE_MILLION = 1000000;
    public static final double _ONE_BILLION = 1000000000;
    public static final String _UNASSIGNED = "Unassigned";
    public static final String _SIMPLEDATE_FORMAT = "dd MMM yyyy HH:mm:ss";
    public static final String _SIMPLEDATE_FORMAT_YYYYMMDD = "yyyyMMdd";
    public static final String _SIMPLEDATE_FORMAT_FULL_MONTH = "dd MMMM yyyy HH:mm:ss";
    public static final String _SIMPLEDATE_FORMAT_FULL_MONTH_NO_TIME = "dd MMMM yyyy";
    public static final String _VRS_DATE_FORMAT = "d MMMM yyyy 00:00:00";
    public static final String _INPUT_DATE_FORMAT = "dd-MMM-yyyy";
    public static final String _DEFAULT_END_OF_YEAR_DATE = "20070405"; //Only date and month matters;
    public static final String _DEFAULT_INCEPTION_DATE = "19000101";

    static final String SPLIT_CHARS = "(?=[\\:;])" + "|(?=[ ;])" + "|(?=[\\-;])" + "|(?=[0;])" + "|(?=[1;])" + "|(?=[2;])" + "|(?=[3;])"
            + "|(?=[4;])" + "|(?=[5;])" + "|(?=[6;])" + "|(?=[7;])" + "|(?=[8;])" + "|(?=[9;])" + "|(?=[\\(;])" + "|(?=[\\);])" + "|(?=[\\&;])"
            + "|(?=[\\';])" + "";

    private static Log logManager = LogFactory.getLog(DateHelper.class);
    public static byte intType = 0;
    public static byte stringType = 1;

    /**
     * return the current date in yyyymmdd format
     * @return
     */
    public static String getCurrentDateyyyymmdd() {

		Calendar cal = Calendar.getInstance();
		String dd = String.format("%02d", cal.get(Calendar.DAY_OF_MONTH));
		String mm = String.format("%02d", cal.get(Calendar.MONTH) + 1);
		String yy = String.valueOf(cal.get(Calendar.YEAR));
		
		String today = yy + mm + dd;
		return today;
    }
    /**
     * return the current date in yyyy-mm-dd format 
     * @return
     */
    public static String getCurrentDateyyyymmdd2() {

		Calendar cal = Calendar.getInstance();
		String dd = String.format("%02d", cal.get(Calendar.DAY_OF_MONTH));
		String mm = String.format("%02d", cal.get(Calendar.MONTH) + 1);
		String yy = String.valueOf(cal.get(Calendar.YEAR));
		
		String today = yy + "-" + mm + "-" + dd;
		return today;
    }
    
    /**
     * return the current date in yyyy-dd-mm format for loading into the database
     * @return
     */
    public static String getCurrentDateyyyyddmm() {

		Calendar cal = Calendar.getInstance();
		String dd = String.format("%02d", cal.get(Calendar.DAY_OF_MONTH));
		String mm = String.format("%02d", cal.get(Calendar.MONTH) + 1);
		String yy = String.valueOf(cal.get(Calendar.YEAR));
		
		String today = yy + "-" + dd + "-" + mm;
		return today;
    	
    }
    public static String getCurrentDateAndTimeyyyymmddhhmmssms() {

		Calendar cal = Calendar.getInstance();
		String dd = String.format("%02d", cal.get(Calendar.DAY_OF_MONTH));
		String mm = String.format("%02d", cal.get(Calendar.MONTH) + 1);
		String yy = String.valueOf(cal.get(Calendar.YEAR));
		String h = String.format("%02d", cal.get(Calendar.HOUR_OF_DAY));
		String m = String.format("%02d", cal.get(Calendar.MINUTE));
		String s = String.format("%02d", cal.get(Calendar.SECOND));
		String ms = String.format("%02d", cal.get(Calendar.MILLISECOND));
		
		String today = yy + mm + dd + h + m + s + ms;
		return today;
    }
    public static String getCurrentDateAndTimeyyyymmdd() {

		Calendar cal = Calendar.getInstance();
		String dd = String.format("%02d", cal.get(Calendar.DAY_OF_MONTH));
		String mm = String.format("%02d", cal.get(Calendar.MONTH) + 1);
		String yy = String.valueOf(cal.get(Calendar.YEAR));
		String h = String.format("%02d", cal.get(Calendar.HOUR_OF_DAY));
		String m = String.format("%02d", cal.get(Calendar.MINUTE));
		String s = String.format("%02d", cal.get(Calendar.SECOND));
		
		String today = yy + mm + dd + '-' + h + m + s;
		return today;
    }
    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * Formats the datetime string with 'T' dividing date and time to create the
     * datetime in format that is acceptable by VRS ("dd mmm yyyy hh:mm:ss")
     */
    public static String getDateInVRSFormat(String val, int col, String colType) {
        String dateInVRSFormat = val;

        //If datetime, rearrange the date value structure
        if (colType.equals("datetime")) {
            //If expected format like 2008-01-09T12:39:36
            if (val.indexOf("T") > 0) {
                try {
                    //Get month 'str' value
                    String month;
                    switch (Integer.parseInt(val.substring(5, 7))) {
                    case 1:
                        month = "Jan";
                        break;
                    case 2:
                        month = "Feb";
                        break;
                    case 3:
                        month = "Mar";
                        break;
                    case 4:
                        month = "Apr";
                        break;
                    case 5:
                        month = "May";
                        break;
                    case 6:
                        month = "Jun";
                        break;
                    case 7:
                        month = "Jul";
                        break;
                    case 8:
                        month = "Aug";
                        break;
                    case 9:
                        month = "Sep";
                        break;
                    case 10:
                        month = "Oct";
                        break;
                    case 11:
                        month = "Nov";
                        break;
                    case 12:
                        month = "Dec";
                        break;
                    default:
                        month = "Jan";
                        break;
                    }
                    dateInVRSFormat = val.substring(8, 10) + " " + month + " " + val.substring(0, 4) + " " + val.substring(11);
                } catch (Exception e) {
                    logManager.error("Error setting the date value in the special format with 'T' in the middle. The value that cannot be set is "
                            + val);
                    dateInVRSFormat = "1 Jan 1900 00:00:00";
                }
            }
        }

        return dateInVRSFormat;
    }

    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * Compares two dates to determine if one is after the other.
     * 
     * @param date1
     * @param date2
     * @return Compares two dates to determine if one is after the other.
     * @throws VermilionNonFatalException
     */
    public static boolean isDateAfterDate(Date date1, Date date2) {

        Calendar calendarDate1 = new GregorianCalendar();
        calendarDate1.setTime(date1);
        calendarDate1.set(Calendar.HOUR, 0);
        calendarDate1.set(Calendar.MINUTE, 0);
        calendarDate1.set(Calendar.SECOND, 0);
        calendarDate1.set(Calendar.MILLISECOND, 0);

        Calendar calendarDate2 = new GregorianCalendar();
        calendarDate2.setTime(date2);
        calendarDate2.set(Calendar.HOUR, 0);
        calendarDate2.set(Calendar.MINUTE, 0);
        calendarDate2.set(Calendar.SECOND, 0);
        calendarDate2.set(Calendar.MILLISECOND, 0);

        return calendarDate1.after(calendarDate2);
    }

    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * As isDateAfterDate but handles nulls pointer exceptions (returns false if
     * exceptions)
     * 
     * @param date1
     * @param date2
     * @return
     */
    public static boolean isNullDateAfterDate(Date date1, Date date2) throws Exception {
        try {
            return isDateAfterDate(date1, date2);
        } catch (NullPointerException e) {
            return false;
        } catch (Exception e) {
            throw new Exception("Unexpected exception in isNullDateAfterDate " + date1 + ", " + date2 + ", (" + e.toString() + ")");
        }
    }

    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * 
     * @param date1
     * @param date2
     * @return
     */
    public static boolean isNullDateOnOrAfterDate(Date date1, Date date2) throws Exception {
        try {
            Calendar calendarDate1 = new GregorianCalendar();
            calendarDate1.setTime(date1);
            calendarDate1.set(Calendar.HOUR, 0);
            calendarDate1.set(Calendar.MINUTE, 0);
            calendarDate1.set(Calendar.SECOND, 0);
            calendarDate1.set(Calendar.MILLISECOND, 0);

            Calendar calendarDate2 = new GregorianCalendar();
            calendarDate2.setTime(date2);
            calendarDate2.set(Calendar.HOUR, 0);
            calendarDate2.set(Calendar.MINUTE, 0);
            calendarDate2.set(Calendar.SECOND, 0);
            calendarDate2.set(Calendar.MILLISECOND, 0);

            if (calendarDate1.equals(calendarDate2)) {
                return true;
            } else {
                if (calendarDate1.after(calendarDate2)) {
                    return true;
                } else {
                    return false;
                }
            }
        } catch (NullPointerException e) {
            return false;
        } catch (Exception e) {
            throw new Exception("Unexpected exception in isNullDateAfterDate " + date1 + ", " + date2 + ", (" + e.toString() + ")");
        }
    }

    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * 
     * @param date1
     * @param date2
     * @return
     */
    public static boolean isNullDateOnOrBeforeDate(Date date1, Date date2) throws Exception {
        try {
            Calendar calendarDate1 = new GregorianCalendar();
            calendarDate1.setTime(date1);
            calendarDate1.set(Calendar.HOUR, 0);
            calendarDate1.set(Calendar.MINUTE, 0);
            calendarDate1.set(Calendar.SECOND, 0);
            calendarDate1.set(Calendar.MILLISECOND, 0);

            Calendar calendarDate2 = new GregorianCalendar();
            calendarDate2.setTime(date2);
            calendarDate2.set(Calendar.HOUR, 0);
            calendarDate2.set(Calendar.MINUTE, 0);
            calendarDate2.set(Calendar.SECOND, 0);
            calendarDate2.set(Calendar.MILLISECOND, 0);

            if (calendarDate1.equals(calendarDate2)) {
                return true;
            } else {
                if (calendarDate1.before(calendarDate2)) {
                    return true;
                } else {
                    return false;
                }
            }
        } catch (NullPointerException e) {
            return false;
        } catch (Exception e) {
            throw new Exception("Unexpected exception in isNullDateAfterDate " + date1 + ", " + date2 + ", (" + e.toString() + ")");
        }
    }

    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * 
     * @param date1
     * @param date2
     * @return
     */
    public static boolean isNullDateBeforeDate(Date date1, Date date2) throws Exception {
        try {
            Calendar calendarDate1 = new GregorianCalendar();
            calendarDate1.setTime(date1);
            calendarDate1.set(Calendar.HOUR, 0);
            calendarDate1.set(Calendar.MINUTE, 0);
            calendarDate1.set(Calendar.SECOND, 0);
            calendarDate1.set(Calendar.MILLISECOND, 0);

            Calendar calendarDate2 = new GregorianCalendar();
            calendarDate2.setTime(date2);
            calendarDate2.set(Calendar.HOUR, 0);
            calendarDate2.set(Calendar.MINUTE, 0);
            calendarDate2.set(Calendar.SECOND, 0);
            calendarDate2.set(Calendar.MILLISECOND, 0);

            if (calendarDate1.before(calendarDate2)) {
                return true;
            } else {
                return false;
            }
        } catch (NullPointerException e) {
            return false;
        } catch (Exception e) {
            throw new Exception("Unexpected exception in isNullDateAfterDate " + date1 + ", " + date2 + ", (" + e.toString() + ")");
        }
    }

    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * Returns the next month start date.
     * 
     * @param _currdate
     *                   current date
     * @param _bflagYearStart
     *                   flag to indicate dealing with end of year
     * @return the next months starting date
     */
    public static Date getNextMonthStart(Date _currdate, boolean _bflagYearEnd) {

        Date startDate = null;

        Calendar calendarDate = new GregorianCalendar();
        calendarDate.setTime(_currdate);
        int month = calendarDate.get(Calendar.MONTH);

        if (_bflagYearEnd && month == Calendar.DECEMBER) {
            return null;
        }

        // add one month to the current date and then set the date to be the
        // first day of the month
        calendarDate.add(Calendar.MONTH, 1);
        calendarDate.set(Calendar.DATE, 1);
        startDate = calendarDate.getTime();

        return startDate;
    }

    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * Returns start date of the specified term. 
     * 
     * @param endDate
     *                   Term end date
     * @param term
     *                   The code of the Term (QTD,MTD,FQTD etc)
     * @param fundFinancialYearEndDate
     *                   Whether the end date of previous period is required rather
     *                   than the first date of this period
     * @return Returns start date.
     * @throws VermilionNonFatalException
     */
    public static Date getStartDateOfTerm(Date endDate, String term, Date fundFinancialYearEndDate, boolean isToGetTheLastDateOfPreviousTermInstead)
            throws VermilionNonFatalException {

        Calendar calendarDate = new GregorianCalendar();
        calendarDate.setTime(endDate);
        Calendar fyeDate = new GregorianCalendar();
        if (fundFinancialYearEndDate != null) {
            fyeDate.setTime(fundFinancialYearEndDate);
        }

        int month = calendarDate.get(Calendar.MONTH);
        Date startDate = null;

        if (term.equals(TERM_QTD)) {
            calendarDate.set(Calendar.DATE, 1);
            switch (month) {
            case Calendar.JANUARY:
            case Calendar.FEBRUARY:
            case Calendar.MARCH:
                calendarDate.set(Calendar.MONTH, Calendar.JANUARY);
                break;
            case Calendar.APRIL:
            case Calendar.MAY:
            case Calendar.JUNE:
                calendarDate.set(Calendar.MONTH, Calendar.APRIL);
                break;
            case Calendar.JULY:
            case Calendar.AUGUST:
            case Calendar.SEPTEMBER:
                calendarDate.set(Calendar.MONTH, Calendar.JULY);
                break;
            case Calendar.OCTOBER:
            case Calendar.NOVEMBER:
            case Calendar.DECEMBER:
                calendarDate.set(Calendar.MONTH, Calendar.OCTOBER);
                break;
            }
        } else if (term.equals(TERM_WTD)) {
            calendarDate.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
        } else if (term.equals(TERM_MTD)) {
            calendarDate.set(Calendar.DATE, 1);
        } else if (term.equals(TERM_6MTD)) {
            calendarDate.add(Calendar.MONTH, -6);
            calendarDate.set(Calendar.DATE, 1);

            switch (month) {
            case Calendar.JANUARY:
            case Calendar.FEBRUARY:
            case Calendar.MARCH:
            case Calendar.APRIL:
            case Calendar.MAY:
            case Calendar.JUNE:
                calendarDate.set(Calendar.MONTH, Calendar.JANUARY);
                break;
            case Calendar.JULY:
            case Calendar.AUGUST:
            case Calendar.SEPTEMBER:
            case Calendar.OCTOBER:
            case Calendar.NOVEMBER:
            case Calendar.DECEMBER:
                calendarDate.set(Calendar.MONTH, Calendar.JULY);
                break;
            }
        } else if (term.equals(TERM_FQTD)) {
            //Financial Quarter to Date (FQTD) => Start Date =
            //	a) if end date is in the Financial Year End (FYE) month then =>
            //	If end date is after the FYE date then return FYE date + 1 day
            //	Else return QTD of (end date less ~30 days)
            //	b) if end date is not in FYE month then return QTD of end date

            if (month == fyeDate.get(Calendar.MONTH)) {
                if (calendarDate.get(Calendar.DAY_OF_MONTH) > fyeDate.get(Calendar.DAY_OF_MONTH)) {
                    calendarDate.set(Calendar.DAY_OF_MONTH, fyeDate.get(Calendar.DAY_OF_MONTH) + 1);
                } else {
                    //Subtract one month from the end date
                    int mth = calendarDate.get(Calendar.MONTH);
                    if (mth == Calendar.JANUARY) {
                    	calendarDate.set(Calendar.MONTH, Calendar.DECEMBER);
                    	calendarDate.set(Calendar.YEAR, calendarDate.get(Calendar.YEAR)-1);
                    } else {
                    	calendarDate.set(Calendar.MONTH, calendarDate.get(Calendar.MONTH)-1);
                    }
                    //Return normal QTD for the same date in the previous month
                    return getStartDateOfTerm(endDate, TERM_QTD, fundFinancialYearEndDate, isToGetTheLastDateOfPreviousTermInstead);
                }
            } else {
                //If not in the Financial Year Month End Month, return normal
                // QTD
                return getStartDateOfTerm(endDate, TERM_QTD, fundFinancialYearEndDate, isToGetTheLastDateOfPreviousTermInstead);
            }
        } else if (term.equals(TERM_FMTD)) {
            if (month == fyeDate.get(Calendar.MONTH)) {
                if (calendarDate.get(Calendar.DAY_OF_MONTH) > fyeDate.get(Calendar.DAY_OF_MONTH)) {
                    //If in the FYE month and after the FYE date
                    calendarDate.set(Calendar.DAY_OF_MONTH, fyeDate.get(Calendar.DAY_OF_MONTH) + 1);
                } else {
                    //If in the FYE month and before the FYE date -- move to the 1st date of the previous month
                    calendarDate.add(Calendar.MONTH, -1);
                    calendarDate.set(Calendar.DATE, 1);
                }
            } else {
                //If not in the Financial Year Month End Month, treat as normal MTD
                calendarDate.set(Calendar.DATE, 1);
            }
        } else {
            throw new VermilionNonFatalException("Unknown term type " + term, 0);
        }

        //Return the last date of the previous period if required
        if (isToGetTheLastDateOfPreviousTermInstead) {
            calendarDate.add(Calendar.DATE, -1);
        }

        startDate = calendarDate.getTime();

        return startDate;
    }


	 // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * Returns the previous month end date.
     * 
     * @param endDate
     *                   Current end date
     * @param flagYearStart
     *                   Flag year start.
     * @return Returns the previous month end date.
     */
    public static Date getPreviousTermEnd(Date endDate, String strTermCode) {
        Date startDate = null;
        try {
            Calendar calendarDate = new GregorianCalendar();
            calendarDate.setTime(getStartDateOfTerm(endDate, strTermCode, false));

            calendarDate.set(Calendar.DATE, 1);
            calendarDate.add(Calendar.DATE, -1);

            startDate = calendarDate.getTime();
        } catch (VermilionNonFatalException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return startDate;
    }
	/**
	 * Returns start date of the specified term.
	 * 
	 * @param endDate Term end date
	 * @param term Type of Term.
	 * @return Returns start date.
	 * @throws VermilionNonFatalException
	 */
	public static Date getStartDateOfTerm(Date endDate, String term, boolean isPreviousTermEndDate) throws VermilionNonFatalException {
		return getStartDateOfTerm(endDate, term, null,isPreviousTermEndDate);
	}
    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * Returns start date of the specified term.
     * 
     * @param endDate
     *                   Term end date
     * @param term
     *                   Type of Term.
     * @return Returns start date.
     * @throws VermilionNonFatalException
     */

    public static Date getStartDateOfTerm(ParameterRecord dateRecord, ParameterRecord termRecord) throws VermilionNonFatalException {
        String quarterType = TERM_QTD;
        Date termStartDate = null;

        String termCode = termRecord.getFieldValue(ENTITYF_TERM_CODE);
        termCode = (termCode.equals(TERM_QTD) ? quarterType : termCode);
        logManager.debug("Term Code : " + termCode);

        String endDate = dateRecord.getFieldValue(ENTITY_CD_DATE);
        logManager.debug("End Date : " + endDate);

        try {
            SimpleDateFormat inputDateFormat = new SimpleDateFormat(_SIMPLEDATE_FORMAT_YYYYMMDD);
            Date endCalDate = inputDateFormat.parse(endDate);

            // Get term start date.
            termStartDate = getStartDateOfTerm(endCalDate, termCode, false);
        } catch (ParseException e) {
            logManager.error("Error occured processing dates.", e);
            e.printStackTrace();
        }
        return termStartDate;
    }
    
 	// from dd mmmm yyyy to yyyymmdd
 	public static String convertDate(String inDate) throws ParseException {
 		return convertDate(inDate,"yyyyMMdd");
 	}
	// from dd mmmm yyyy to any format
 	public static String convertDate(String inDate, String outDateFormat) throws ParseException {
 		SimpleDateFormat inputDateFormat = new SimpleDateFormat("dd MMMM yyyy");
 		SimpleDateFormat outputDateFormat = new SimpleDateFormat(outDateFormat);
		   
 		Date endCalDate = inputDateFormat.parse(inDate);
 		return outputDateFormat.format(endCalDate);
 	}

 

 	//this method does not care about the specific day of the month in the dates
 	// given. For example, in the same year, July 29 and August 2 will have 
 	// 1 month between them, but July 3rd and July 25th will have 0 
 	// months between them.
 	
 	public static int monthsBetween(String date1, String format1, String date2, String format2)
 		throws ParseException{

 		//get the month and year of first date as ints
		SimpleDateFormat dateformat1 = new SimpleDateFormat(format1);
		Calendar calendar1 = new GregorianCalendar();
		calendar1.setTime(dateformat1.parse(date1));
		int year1 = calendar1.get(Calendar.YEAR);
		int month1 = calendar1.get(Calendar.MONTH);
		
		//get the month and year of second date as ints
		SimpleDateFormat dateformat2 = new SimpleDateFormat(format2);
		Calendar calendar2 = new GregorianCalendar();
		calendar2.setTime(dateformat2.parse(date2));
		int year2 = calendar2.get(Calendar.YEAR);
		int month2 = calendar2.get(Calendar.MONTH);

		int possibleMonthsBetween = 12*(year2-year1)+(month2-month1);
		calendar1.add(Calendar.MONTH, possibleMonthsBetween);
		
		if(calendar1.compareTo(calendar2)<=0){
			return possibleMonthsBetween;
		} else{
			return possibleMonthsBetween-1;
		}
 	}
 	
    public static String addMonths(String date1, String datePart, String addAmount) {
    	return addMonths(date1, datePart, Integer.parseInt(addAmount));
    }
    
    public static String addMonths(String date1, String datePart, int addAmount) {
    	return addMonths(getYyyymmdd(date1), datePart, addAmount);
    }
    
 	// from dd mmmm yyyy to yyyymmdd
 	public static Date getYyyymmdd(String inDate) {
 		SimpleDateFormat inputDateFormat = new SimpleDateFormat("yyyyMMdd");
		   
 		try {
			return inputDateFormat.parse(inDate);
		} catch (ParseException e) {
            logManager.error("Error converting the date from a string to a date type : " + inDate);
            return null;
		}
 	}
 	
    public static String addMonths(Date date1, String datePart, int addAmount) {
        Calendar calendarDate = new GregorianCalendar();
        calendarDate.setTime(date1);

        if(datePart.equalsIgnoreCase("dd")) {
            calendarDate.add(Calendar.DAY_OF_MONTH, addAmount);
        }
        if(datePart.equalsIgnoreCase("mm")) {
            calendarDate.add(Calendar.MONTH, addAmount);
        }
        if(datePart.equalsIgnoreCase("yy")) {
            calendarDate.add(Calendar.YEAR, addAmount);
        }
        
		String mm = String.format("%02d", calendarDate.get(Calendar.MONTH) + 1);
		
        return mm;
    }
}

