package com.vermilion.vrs.common;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.vermilionsoftware.vrs.common.exception.VermilionNonFatalException;
import com.vermilionsoftware.vrs.dgs.api.ParameterField;
import com.vermilionsoftware.vrs.dgs.api.ParameterList;
import com.vermilionsoftware.vrs.dgs.api.ParameterRecord;
import com.vermilionsoftware.vrs.dgs.api.RecordSet;
import com.vermilionsoftware.vrs.dgs.api.RecordSetList;
import com.vermilionsoftware.vrs.dgs.api.WarningMessage;

public class StringHelper {
	private static Log logManager = LogFactory.getLog(StringHelper.class);
    // -----------------------------------------------------------------------------------------------------------------------

    public static void copyWarningMessages(RecordSetList inputRecordSetList, RecordSet outputRecordSet) throws VermilionNonFatalException {
 		    
 	    ArrayList<String> warnings = new ArrayList<String>();
 	    ArrayList<String> cautions = new ArrayList<String>();

 	    int warningCount = outputRecordSet.getWarningMessageCount();
 		for(int w=0; w<warningCount; w++) {
 	        warnings.add(outputRecordSet.getWarningMessageText(w));
 	    }
 		
 		for(int l=0; l<inputRecordSetList.getRecordSetCount(); l++) {
 			RecordSet inputRecordSet = inputRecordSetList.getRecordSet(l);
 			warningCount = inputRecordSet.getWarningMessageCount();
 			int renderingCount = inputRecordSet.getRenderingInstructionCount();
 			int cautionCount = inputRecordSet.getCautionMessageCount();

 			if (cautionCount > 0) {
 				for (int c = 0 ; c < cautionCount ; c++) {
 					if ( ! cautions.contains( inputRecordSet.getCautionMessageText(c) ) ) {
 					    outputRecordSet.addCautionMessage(inputRecordSet.getCautionMessage(c));
 					    cautions.add(inputRecordSet.getCautionMessageText(c));
 					}
 				}
 	        }                

            // Copy any warning messages from the input record set into the output record set.
 			if (warningCount > 0) {
 				for (int warningIndex = 0 ; warningIndex < warningCount ; warningIndex++) {
 					//customLog("[copyWarningMessages] message text = " + inputRecordSet.getWarningMessage(warningIndex).getMessageText());
 					if ( ! warnings.contains( inputRecordSet.getWarningMessageText(warningIndex) ) ) {
 					    outputRecordSet.addWarningMessage(inputRecordSet.getWarningMessage(warningIndex));
 					    warnings.add(inputRecordSet.getWarningMessageText(warningIndex));
 					}
 				}
 	        }                
 	    
 	        if (renderingCount > 0) {
 	            for (int renderingIndex = 0 ; renderingIndex < renderingCount ; renderingIndex++) {
 					//customLog("[copyWarningMessages] rendering instruction xml = " + inputRecordSet.getRenderingInstruction(renderingIndex).getInstructionXML());
 	            	outputRecordSet.addRenderingInstruction(inputRecordSet.getRenderingInstruction(renderingIndex));
 	            }
 	        }
 	     }

 	}
    // ------------------------------------------------------------------------------------------------------------------------------
    public static String getProperty(String propertiesText, String propertyName, String defaultValue) {
    	// load the properties if not already done
    	Properties properties = new Properties();
    	
        try {
            ByteArrayInputStream reader = new ByteArrayInputStream(propertiesText.getBytes());
            properties.load(reader);
        } catch (IOException e) {
        	e.printStackTrace();
        }
    	
        String value = null;
        value = properties.getProperty(propertyName);

        if (value == null) {
        	value = defaultValue;
        }
        return value;
    }
    
    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * Main method for replacing substitution variables in input filenames, 
     * e.g. String fullname = getResolveSubVars("$_Account.Full Name$")
     * 
     * @param filePath
     * @param parameterList
     * @param ignoreEntityNulls
     * @return
     */
    public static String getResolveSubVars(String path, ParameterList parameterList, boolean ignoreEntityNulls) {
        String result = path;
        String entityNm = "";
        ArrayList<String> pl = parameterList.getParameterNames();


        if (result.indexOf("$_") > -1) {
	        for (int p = 0; p < parameterList.getParameterCount(); p++) {
	            try {
	                entityNm = (String) pl.get(p);
	                if (result.indexOf("$_") > -1) {
	                    // loop through the parameter fields regardless of the
	                    // entity name as the entity may include reference
	                    // entities....
	                    result = getParameterReplacement(result, parameterList.getParameterRecord(p), ignoreEntityNulls);
	                }
	            } catch (Exception e) {
	                if (e.toString().indexOf("Unable to create") > -1 && e.toString().indexOf("No data found for specified parameters") > -1) {
	                    logManager.error("::Problems processing Parameter record " + p + " (" + entityNm
	                            + "); is this parameter chained from an outer datasource?");
	                } else {
	                    logManager.error("::Problems processing Parameter " + p + " .." + entityNm
	                            + ".. in the getResolveSubVars method of GeneralHelper custom class. (" + e.toString() + ")");
	                }
	            }
	        }
        }
        return result;
    }
    

    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * 
     * @param result
     * @param pr
     * @param ignoreEntityNulls
     * @return
     */
    public static String getParameterReplacement(String result, ParameterRecord pr, boolean ignoreEntityNulls) {
        return getParameterReplacement(result, pr, "", ignoreEntityNulls);
    }

    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * Method used to replace substitution strings (e.g. $_Date.SysDate$) with
     * their actual values
     * 
     * @param result
     * @param pr
     * @param hierarchyTrace
     * @param ignoreEntityNulls
     * @return
     */
    private static String getParameterReplacement(String result, ParameterRecord pr, String hierarchyTrace, boolean ignoreEntityNulls) {
        String errorMsg = "";
        try {
            String entityNm = "";
            String fieldNm = "";
            String entityValue = "";
            String dtFormat = "d MMM yyyy";
            String testParam = "";
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat(dtFormat);
            SimpleDateFormat outputDateFormat = new SimpleDateFormat("yyyyMMdd");

            entityNm = pr.getParameterName();
            ArrayList<String> fieldNms = pr.getFieldNames();
            Date date;

            for (int f = 0; f < pr.getFieldCount(); f++) {
                errorMsg = "processing " + result + " and looking at " + entityNm + "." + fieldNms.get(f) + " (" + f + " of " + pr.getFieldCount()
                        + ")";
                fieldNm = (String) fieldNms.get(f);

                if (pr.getField(f).isParameterRecord()) {
                	if (pr.getField(f).isMultiParameterRecord()){
                		//nothing to do here
                	}
                	else if (pr.getField(f).isNull()) {
                        //    				if ( pr.isFieldNull(f) || pr.isFieldEmpty(f) ) {
                        entityValue = "";
                    } else {
                        try {
                            testParam = "$_" + entityNm + "." + fieldNm + "$";
                            if (result.indexOf(testParam) > -1) {

                                ParameterField pfReferenceField = pr.getField(fieldNm);
                                result = getStringReplacement(result, testParam, pfReferenceField.getParameterRecord().getKeyValue()); // We don't know which field is the description - so just return the first field

                            } else {
                                result = getParameterReplacement(result, pr.getField(f).getParameterRecord(), entityNm + ">>", ignoreEntityNulls);
                            }
                        } catch (Exception e) {
                            // Ignore this as it might be an Unable to Create
                            // ParameterRecord because the isFieldNull() check
                            // doesn't handle reference fields
                            //logManager.error("Exception in " + hierarchyTrace + " [" + " trace1.1genhelper " + e.toString() + ") for " + fieldNm
                            //      + "/" + result);
                        }
                    }
                } else {
                    if (pr.isFieldNull(f)) {
                        if (ignoreEntityNulls) {
                            entityValue = "";
                        } else {
                            entityValue = "null";
                        }
                    } else {
                        entityValue = pr.getFieldValue(fieldNm);
                    }

                    if (entityValue.trim().equals("")) {
                        if (!ignoreEntityNulls) {
                            entityValue = "... [no value stored for '" + fieldNm + "' (please check the " + entityNm + " entity values screen)] ...";
                        } else {
                            entityValue = "";
                        }
                    } else {

                        if (pr.getField(fieldNm).getFieldType().equals("datetime")) {
                            try {
                                testParam = "$_" + entityNm + "." + fieldNm + "$";
                                if (result.indexOf("/formatdate(" + testParam) > -1) {
                                	while(result.indexOf("/formatdate(" + testParam) > -1){
	                                    int i1 = result.indexOf("/formatdate(" + testParam);
	                                    String newFormat = result.substring(i1);
	                                    int i2 = result.indexOf(")", i1);
	
	                                    newFormat = newFormat.substring(newFormat.indexOf(",") + 1);
	                                    newFormat = newFormat.substring(0, newFormat.indexOf(")"));
	                                    
	                                    newFormat = getStringReplacement(newFormat, "\"", "");
	                                    newFormat = getStringReplacement(newFormat, "m", "M");
	                                    
	                                   	String oldFormat = pr.getField(fieldNm).getFieldDisplayFormat();
	                                    oldFormat = getStringReplacement(oldFormat, "m", "M");

	                                    
	                                    SimpleDateFormat sdf1 = new SimpleDateFormat(oldFormat);
	                                    date = sdf1.parse(entityValue);
	                                    
	                                    SimpleDateFormat sdf2 = new SimpleDateFormat(newFormat);
	                                    String newEntityValue = sdf2.format(date);
	                                    result = getStringReplacement(result, result.substring(i1, i2 + 1), newEntityValue);
                                	}
                                } else {
                                    if (result.indexOf(testParam) > -1) {

                                        // if it's a datetime - then .....
                                        errorMsg = "Formatting Date " + entityValue;
                                        if (pr.getField(fieldNm).getFieldDisplayFormat().equals("dd.mm.yyyy")) {
                                            SimpleDateFormat sdf1 = new SimpleDateFormat("dd.MM.yyyy");
                                            date = sdf1.parse(entityValue);
                                        } else {
                                            date = simpleDateFormat
                                                    .parse(getStringReplacement(getStringReplacement(entityValue, "-", " "), "/", " "));
                                        }
                                        entityValue = outputDateFormat.format(date);

                                    } else {
                                        //This datetime field does not appear as a parameter in the string result.
                                    }
                                }
                            } catch (Exception e) {
                                // Ignore this as it might be an Unable to
                                // Create ParameterRecord because the
                                // isFieldNull() check doesn't handle reference
                                // fields
                                //logManager.error("Exception in " + hierarchyTrace + " [" + " trace1.1genhelper " + e.toString() + ") for " + fieldNm
                                //        + "/" + result, e);
                            }

                        }
                    }
                    result = getStringReplacement(result, "$_" + entityNm + "." + fieldNm + "$", entityValue);
                }
            }
        } catch (Exception e) {
            result = "Problems processing " + pr.getParameterName() + " (" + errorMsg
                    + ") in the getParameterReplacement method of GeneralHelper custom class. (" + e.toString() + ")";
        }
        return result;
    }
    
    
    // --------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * getStringReplacement - used to recursively replace strings with other
     * strings (standard Java methods are not up the job with $ and &
     * characters)
     * 
     * @param result
     * @param replaceThis
     * @param withThis
     * @return
     */
    public static String getStringReplacement(String result, String replaceThis, String withThis) {
        int idx = 0;
        int maxtries = 9;
        int replacementLen = replaceThis.length();

        while (idx != -1 && maxtries-- > 0) {
            idx = result.indexOf(replaceThis);
            if (idx != -1) {
                StringBuffer ret = new StringBuffer(result);
                ret.replace(idx, idx + replacementLen, withThis);
                while ((idx = result.lastIndexOf(replaceThis, idx - 1)) != -1 && maxtries-- > 0) {
                    ret.replace(idx, idx + replacementLen, withThis);
                }
                idx = 0;
                result = ret.toString();
            }
        }
        return result;
    }
    

	public static String getEntityValuesInString(RecordSet rst, ParameterList parameterList)
	{
	    String entityValues = "";
	    String tmp;
	    try
	    {
	        tmp = getResolveSubVars("$_Fund.List Name$", parameterList, true); //  + " / ";
	        if ( tmp.indexOf("$_") == -1 ) { // If it was resolved then
	            entityValues = tmp;
	        }
	        tmp = getResolveSubVars("$_Date.SysDate$", parameterList, true);
	        if ( tmp.indexOf("$_") == -1 ) { // If it was resolved then
	            if ( entityValues.equals("") ) {
	                if ( tmp.trim().length() == 8 ) {
	                    entityValues = tmp.substring(6,8)+"."+tmp.substring(4,6)+"."+tmp.substring(0,4);
	                }
	                else {
	                    entityValues = tmp;
	                }
	            }
	            else {
	                if ( tmp.trim().length() == 8 ) {
	                    entityValues += " / "+ tmp.substring(6,8)+"."+tmp.substring(4,6)+"."+tmp.substring(0,4);
	                }
	                else {
	                    entityValues += " / " + tmp;
	                }
	              
	            }
	        }
	        //tmp = GeneralHelper.getResolveSubVars("$_Term.List Name$", parameterList, true) + " / ";
	        //tmp = GeneralHelper.getResolveSubVars("$_Pricing Basis.List Name$", parameterList, true);
	        
	        if ( ! entityValues.equals("") ) {
	            entityValues = "(for " + entityValues + ")";
	        }
	    }
	    catch (Exception e)
	    {
			WarningMessage warningMessage = new WarningMessage(0, "Getting Entity Values for Warning", WarningMessage.DONT_OUTPUT, 
				"Error in getEntityValuesInString(): cannot obtain some entity valies (out of fund name, term, mid/bid, date). Error text:" + e.getMessage());
			rst.addWarningMessage(warningMessage);
	    }
	    return entityValues;
	}



	
	/**
	 * Set the snapshot behaviour of the data source
	 * @param outputRecordSet
	 * @param controlRecordSet
	 * @param parameterList
	 */
	public static RecordSet setSnapshotBehaviour(RecordSet outputRecordSet, RecordSet controlRecordSet, ParameterList parameterList){
	    try {
	        // set the snapshot behaviour depending on whether or not the control record set has been signed off
			if ( outputRecordSet.getRowCount() > 0 ) {

			    String strStatusCode;				
			    // check whether the control record set has been signed off
				if (controlRecordSet != null){			    
				    strStatusCode = controlRecordSet.getApprovedStatusCode();
				} else {		    
				    strStatusCode = "A";
				}
	            boolean dataSignedOff = (strStatusCode != null && strStatusCode.equals("A") ? true : false);

	            // check whether the output record set has been signed off if it needs it
	            if(dataSignedOff){ 
		            if  (outputRecordSet.getDataSourceInfo().getSignOffRequired() ) {
		                strStatusCode = controlRecordSet.getApprovedStatusCode();
		                dataSignedOff = (strStatusCode != null && strStatusCode.equals("A") ? true : false);
		            }
	            }
	            
	            if ( dataSignedOff){
					// force snapshots on this record set and the valuation ready record set
					outputRecordSet.setSnapshotBehaviour(RecordSet.SNAPSHOT_FORCE);
				}
				else {
				    outputRecordSet.setSnapshotBehaviour(RecordSet.SNAPSHOT_PREVENT);
					String inputEntityValues = getEntityValuesInString(outputRecordSet, parameterList);
					String messageTx = "The data source: '" + controlRecordSet.getDataSourceName() + "' " + inputEntityValues + " has not been signed off.";
					boolean duplicate=false;
					for(int m=0; m<outputRecordSet.getWarningMessageCount(); m++) {
						if ( outputRecordSet.getWarningMessageText(m).equals(messageTx)) {
							duplicate=true;
						}
					}
					if ( ! duplicate ) {
						outputRecordSet.addWarningMessage(messageTx);
					}
				}
	            
			} else {
			    outputRecordSet.setSnapshotBehaviour(RecordSet.SNAPSHOT_PREVENT);
			}
	        
		} catch (VermilionNonFatalException e) {
		    logManager.error("Error in setSnapshotBehaviour CommonAncestor: " + e.getMessage());
            e.printStackTrace();
        }
		return outputRecordSet;
	}

	/**
	 * Determines if the string is parseable as a numeric value to BigDecimal (one of the most versatile numeric classes)
	 * @param inputData
	 */
    public static boolean isNumeric(String inputData) {
    	try{
    		@SuppressWarnings("unused")
			BigDecimal inputAsDecimal = new BigDecimal(inputData);
    		return true;
    	} catch(NumberFormatException e){
    		return false;
    	}
    }
    
    /**
	 * Determines if the string has any newfangled functions we'd like to see in the core product.
	 */  
    
    
    public static String parseFunctions(String inputString) throws VermilionNonFatalException{
    	String outputString = inputString;
    	try{
    		int fromIndex=outputString.length();
    		while(outputString.lastIndexOf("/",fromIndex-1)>0){
    			fromIndex=outputString.lastIndexOf("/",fromIndex-1);
    			if(outputString.indexOf("(",fromIndex)>0 && outputString.indexOf(")",fromIndex)>outputString.indexOf("(",fromIndex)){
        			String possibleFunction = outputString.substring(fromIndex);
        			String possibleResolution = resolveFunction(possibleFunction);
        			outputString=getStringReplacement(outputString,possibleFunction,possibleResolution);   	
    			}
    		}
    	}catch(Exception E){
    		logManager.error("Error parsing functions in string{"+outputString+"}: +"+E.toString());
    	}
    	
    	return outputString;
    }
    private static String resolveFunction(String inputString) throws VermilionNonFatalException{
    	String outputString = inputString;
    	try{
            int firstBracket = outputString.indexOf('(', 1);
            int lastBracket = outputString.lastIndexOf(')');
            ArrayList<String> argumentList = new ArrayList<String>();
            int parsingPosition = firstBracket+1;
            int maxLoop=100;
            while(parsingPosition<lastBracket){
            	//prevent infinite loops, just in case:
            	maxLoop--;
            	if (maxLoop<0) break;

            	if(outputString.charAt(parsingPosition)==')'){
            		break;
            	}else if(outputString.charAt(parsingPosition)==','){
            		parsingPosition++;
            		continue;
            	}else{
            		if(outputString.charAt(parsingPosition)=='\'' || outputString.charAt(parsingPosition)=='"'){
                		char quote = outputString.charAt(parsingPosition);
                		if(outputString.indexOf(quote,parsingPosition+1)>0){
                			int closeStringPosition = outputString.indexOf(quote,parsingPosition+1);
                			if(outputString.charAt(closeStringPosition+1)==',' || outputString.charAt(closeStringPosition+1)==')'){
                    			String argument = outputString.substring(parsingPosition+1,closeStringPosition);
                    			argumentList.add(argument);
                    			parsingPosition=closeStringPosition+1;
                    			continue;
                			}
                		}
            		}
            		
            		//The argument was not fully enclosed by quotes, so do not treat first quote as a delimeter but as part of argument
            		
        			if(outputString.indexOf(",",parsingPosition+1)>0){
        				int closeStringPosition = outputString.indexOf(",",parsingPosition+1);
            			String argument = outputString.substring(parsingPosition,closeStringPosition);
            			argumentList.add(argument);
            			parsingPosition=closeStringPosition;
        			}else if(outputString.indexOf(")",parsingPosition+1)>0){
        				int closeStringPosition = outputString.indexOf(")",parsingPosition+1);
            			String argument = outputString.substring(parsingPosition,closeStringPosition);
            			argumentList.add(argument);
            			parsingPosition=closeStringPosition;
        			}else{
        				parsingPosition++;
        			}           		
            	}
            }
            String functionSubstring = outputString.substring(0,parsingPosition+1);
            String funcName = outputString.substring(1, firstBracket);
               if (funcName.equalsIgnoreCase("repeat")){
            	   outputString=getStringReplacement(outputString,functionSubstring,processFunctionRepeat(argumentList));
               }

    	}catch(Exception E){
    		logManager.error("Error resolving functions in string{"+outputString+"}: +"+E.toString());   		
    	}
    	return outputString;	
    }
    private static String processFunctionRepeat(ArrayList<String> argumentList) throws VermilionNonFatalException{
    	String outputString="";
    	try{
    		String stringToRepeat = argumentList.get(0);
    		int repeatCount = Integer.parseInt(argumentList.get(1));
    		for(int i=0;i<repeatCount;i++){
    			outputString=outputString+stringToRepeat;
    		}
		}catch(Exception E){
			logManager.error("Error resolving 'repeat' function:"+E.toString());   		
		}
	return outputString;	   	
    }
}

